// Adapted from original model "base.stan" by Flaxman et al
// https://github.com/ImperialCollegeLondon/covid19model


data {
  int<lower=1> K; //number of mixture categories
  real BOUNDARIES[2*K]; //boundaries of the uniform distributions for the alpha
  int <lower=1> M; // number of regions
  int <lower=1> N0; // number of days for which to impute infections
  int <lower=1> N2; // days of observed data + # of days to forecast
  // int cases[N2,M]; // reported cases
  // int <lower=1> N[M]; // days of observed data for region m. each entry must be <= N2
  int <lower=1> N_obs; // total number of observations

  int <lower=1, upper=M> mm[N_obs]; // indices of where (which region) a number of deaths has been observed
  int <lower=1, upper=N2> nn[N_obs]; // indices of the day when a number of deaths has been observed

  int <lower=0> deaths_obs[N_obs]; // reported deaths -- the rows with i > N contain -1 and should be ignored

  matrix[N2, M] f; // h * s
  matrix[N2, M] lockdown;
  // matrix[N2, M] covariate2;
  // matrix[N2, M] covariate3;
  // matrix[N2, M] elections;
  // matrix[N2, M] covariate4;
  // matrix[N2, M] covariate5;
  // matrix[N2, M] covariate6;
  int EpidemicStart[M];
  real pop[M];
  real SI[N2]; // fixed pre-calculated SI using empirical data from Neil
}

parameters {
  simplex[K] theta;          // mixing proportions
  real<lower=0> mu[M]; // intercept for Rt
  // positive_ordered[K] alpha_hier; // sudo parameter for the hierarchical term for alpha
  real alpha[K];
  real<lower=0> kappa;
  real<lower=0> y[M];  // all mixture components share the same values for the initial days
  real<lower=0> phi;
  real<lower=0> tau;
  real <lower=0> ifr_noise[M];
}

transformed parameters {
  matrix[N2, M] prediction[K];
  matrix[N2, M] E_deaths[K];
  matrix[N2, M] Rt[K];
  matrix[N2, M] Rt_adj[K];
  vector[K] log_theta = log(theta);  // cache log calculation
  for(k in 1:K){
    prediction[k] = rep_matrix(0,N2,M);
    E_deaths[k]  = rep_matrix(0,N2,M);
    Rt[k] = rep_matrix(0,N2,M);
    Rt_adj[k] = Rt[k];
  }

  {
    matrix[N2,M] cumm_sum[K] ;
    for(k in 1:K){
      cumm_sum[k] = rep_matrix(0,N2,M);
      //alpha[k] = alpha_hier[k] - ( log(1.05) / 6.0 );
    }

    for(k in 1:K){
      for (m in 1:M){ // for each region
        for (i in 2:N0){ //for each day
          cumm_sum[k][i,m] = cumm_sum[k][i-1,m] + y[m];
        }
        prediction[k][1:N0,m] = rep_vector(y[m],N0); // learn the number of cases in the first N0 days

        Rt[k][,m] = mu[m] * exp(
          lockdown[,m] * (-alpha[k])
          );

        Rt_adj[k][1:N0,m] = Rt[k][1:N0,m];
        for (i in (N0+1):N2) {
          real convolution=0;
          for(j in 1:(i-1)) {
            convolution += prediction[k][j, m] * SI[i-j];
          }
          cumm_sum[k][i,m] = cumm_sum[k][i-1,m] + prediction[k][i-1,m];
          // adjust Rt for population saturation effect
          Rt_adj[k][i,m] = ((pop[m]-cumm_sum[k][i,m]) / pop[m]) * Rt[k][i,m];
          prediction[k][i, m] = Rt_adj[k][i,m] * convolution;
        }

        E_deaths[k][1, m]= 1e-15 * prediction[k][1,m];
        for (i in 2:N2){
          for(j in 1:(i-1)){
            E_deaths[k][i,m] += prediction[k][j,m] * f[i-j,m] * ifr_noise[m];
          }
        }
      }
    }
  }
}

model {
  tau ~ exponential(0.03);
  for (m in 1:M){
    y[m] ~ exponential(1/tau);
  }
  phi ~ normal(0,5);
  kappa ~ exponential(0.5);
  mu ~ normal(3.28, kappa); // citation: https://academic.oup.com/jtm/article/27/2/taaa021/5735319
  for (k in 1:K){
    alpha[k] ~ uniform(BOUNDARIES[(k-1)*2+1], BOUNDARIES[(k-1)*2+2]); 
  }
  ifr_noise ~ normal(1,0.1);

  for (i_obs in 1:N_obs){
    vector[K] lps = log_theta;
    if (nn[i_obs] >= EpidemicStart[mm[i_obs]]) {
      for (k in 1:K){
        lps[k] += neg_binomial_2_lpmf(deaths_obs[i_obs]| E_deaths[k][nn[i_obs], mm[i_obs]], phi);
      }
      target += log_sum_exp(lps);
    }
  }
}
