// Adapted from original model "base.stan" by Flaxman et al
// https://github.com/ImperialCollegeLondon/covid19model


data {
  int <lower=1> M; // number of regions
  int <lower=1> N0; // number of days for which to impute infections
  int <lower=1> N2; // days of observed data + # of days to forecast
  int <lower=1> N_obs; // total number of observations
  
  int <lower=1, upper=M> mm[N_obs];
  int <lower=1, upper=N2> nn[N_obs];
  
  int <lower=0> deaths_obs[N_obs]; // reported deaths -- the rows with i > N contain -1 and should be ignored
  
  matrix[N2, M] f; // h * s
  matrix[N2, M] lockdown;
  matrix[N2, M] postlockdown;
  int EpidemicStart[M];
  real pop[M];
  real SI[N2]; // fixed pre-calculated SI using empirical data from Neil
}

parameters {
  real<lower=0> mu[M]; // intercept for Rt
  real<lower=0> alpha_hier[2]; // sudo parameter for the hier term for alpha
  real<lower=0> kappa;
  real<lower=0> y[M];
  real<lower=0> phi;
  real<lower=0> tau;
  real <lower=0> ifr_noise[M];
}

transformed parameters {
  real alpha[2];
  matrix[N2, M] prediction = rep_matrix(0,N2,M);
  matrix[N2, M] E_deaths  = rep_matrix(0,N2,M);
  matrix[N2, M] Rt = rep_matrix(0,N2,M);
  matrix[N2, M] Rt_adj = Rt;
  
  {
    matrix[N2,M] cumm_sum = rep_matrix(0,N2,M);
    for (i in 1:2){
      alpha[i] = alpha_hier[i] - ( log(1.05) / 6.0 );
    }
    for (m in 1:M){
      for (i in 2:N0){
        cumm_sum[i,m] = cumm_sum[i-1,m] + y[m]; 
      }
      prediction[1:N0,m] = rep_vector(y[m],N0); // learn the number of cases in the first N0 days
      
      Rt[,m] = mu[m] * exp(
        lockdown[,m] * (-alpha[1]) + 
        postlockdown[,m] * (-alpha[2])
        );
        
      Rt_adj[1:N0,m] = Rt[1:N0,m];
      for (i in (N0+1):N2) {
        real convolution=0;
        for(j in 1:(i-1)) {
          convolution += prediction[j, m] * SI[i-j];
        }
        cumm_sum[i,m] = cumm_sum[i-1,m] + prediction[i-1,m];
        // adjust Rt for population saturation effect
        Rt_adj[i,m] = ((pop[m]-cumm_sum[i,m]) / pop[m]) * Rt[i,m];
        prediction[i, m] = Rt_adj[i,m] * convolution;
      }
      
      E_deaths[1, m]= 1e-15 * prediction[1,m];
      for (i in 2:N2){
        for(j in 1:(i-1)){
          E_deaths[i,m] += prediction[j,m] * f[i-j,m] * ifr_noise[m];
        }
      }
    }
  }
}
  
model {
  tau ~ exponential(0.03);
  for (m in 1:M){
    y[m] ~ exponential(1/tau);
  }
  phi ~ normal(0,5);
  kappa ~ normal(0,0.5);
  mu ~ normal(3.28, kappa); // citation: https://academic.oup.com/jtm/article/27/2/taaa021/5735319
  alpha_hier ~ gamma(.1667,1);
  ifr_noise ~ normal(1,0.1);
  
  for (i_obs in 1:N_obs){
    if (nn[i_obs] >= EpidemicStart[mm[i_obs]])
      deaths_obs[i_obs] ~ neg_binomial_2(E_deaths[nn[i_obs], mm[i_obs]], phi);
  }
}

