library(lubridate)


prepare_french_region_dataset <- function(dataset, keep.all=F, post.lockdown=F){
  dataset %>%
    pivot_longer(cols=c(-jour), names_to = "region",values_to = "deaths") %>%
    rename(date=jour) %>%
    filter(!is.na(deaths), keep.all | !(region %in% c("outre.mer", "outre-mer"))) %>%
    mutate(
      region_id=region %>% as.factor() %>% as.integer(),
      date = lubridate::ymd(date),
      # t = decimal_date(date)
    ) %>%
    arrange(date) %>%
    filter(post.lockdown | date <= ymd("2020-05-11"))
}

load_covariates <- function(path = 'data/interventions.csv'){
  read.csv(path, stringsAsFactors = FALSE, h=T) %>%
    select(Country, Type, Date.effective) %>%
    mutate(Type=
        str_replace_all(Type, "\\+", "") %>% 
        str_squish() %>% 
        str_replace_all("[- ]", "_") %>%
        str_to_lower()
    ) %>%
    pivot_wider(names_from = "Type", id_cols = "Country", values_from = "Date.effective") %>%
    filter(Country == "France") %>%
    select(
      schools_universities, 
      public_events, 
      lockdown, 
      )%>%
    mutate_all(as.Date,format="%d.%m.%Y") %>% 
    # interventions may not occur later than lockdown
    mutate_all(~pmin(., lockdown))
}

pad_region_data <- function(region_data, date_min=dmy('31/12/2019') ){
  first_date <- region_data %>% pull(date) %>% min()
  if(first_date > date_min){
    pad_days <- first_date - date_min
    pad_dates <- date_min + days(1:pad_days - 1)
    message(sprintf(" | %d complete records before padding", nrow(region_data)))
    message(sprintf(" | padding %d days before %s", pad_days, first_date))
    padded_data <- data.frame(
      "t" = pad_dates %>% decimal_date(),
      "date" = pad_dates,
      "deaths" = 0,
      stringsAsFactors = F
    )
    region_data <- bind_rows(padded_data, region_data)
  } 
  return(region_data)
}

slice_from_onset <- function(region_data){
  death_cumsum <- region_data %>% pull(deaths) %>% cumsum()
  index_10_deaths <- which(death_cumsum >= 10) %>% first() # also 5
  index_data_start <- index_10_deaths - 30
  start_date <- region_data %>% slice(index_data_start) %>% pull(date)
  sprintf(
    " | 30 days before 10 deaths is %s : day #%d", 
    start_date, index_data_start
  ) %>% message()
  
  region_data %>%
    slice(index_data_start:n()) %>%
    mutate(day_id=as.integer(date-min(date)) + 1)
}

rename_region <- function(region, sep="."){
  region %>%
    str_to_lower() %>%
    str_remove("'") %>%
    str_replace_all("[ -.]", sep) %>%
    iconv(from="UTF-8", to="ASCII//TRANSLIT")
}

prepare_regions_pop_ifr <- function(pop_data, scale=1.0){
  pop_data %>%
    rename(region="Régions") %>%
    rownames_to_column("region_id") %>%
    mutate(
      pop = select(., -region, -region_id, -IFR) %>% rowSums,
      region = rename_region(region),
      IFR = IFR * scale,
      region_id=as.integer(region_id)
    ) %>%
    filter(region_id <= 13) %>%
    select(region, region_id, IFR, pop) %>%
    as_tibble()
}
  
