library(EnvStats) # required for rgammaAlt
library(lubridate)

# various distributions required for modeling
x1 <- rgammaAlt(1e7, 5.1, 0.86) # infection-to-onset distribution
x2 <- rgammaAlt(1e7, 18.8,  0.45) # onset-to-death distribution
ecdf.saved <- ecdf(x1+x2)



make_region_stan_input <- function(region_data, keys, covariates, ifr_pop_data, N2, lockdown_end=T){
  Region <- keys$region
  message(sprintf("STAN inputs : %s", Region))
  # Sample size
  N = nrow(region_data)
  # Region demographics
  region_pop_data <- ifr_pop_data %>% filter(region == Region)
  pop <- region_pop_data$pop
  IFR <- region_pop_data$IFR # IFR is the overall probability of dying given infection
  # f is the probability of dying on day i given infection
  convolution <- function(u) (IFR * ecdf.saved(u))
  f <-  c(
    convolution(1.5) - convolution(0),
    map_dbl(2:N2, ~(convolution(.x+.5) - convolution(.x-.5)))
  )
  # Epidemic start : number of days after initial infection
  EpidemicStart = 31
  
  ### Covariates
  region_first_day <- min(region_data$date)
  day_id_of <- function(day_date){
    1 + as.integer(ymd(day_date) - region_first_day)
  }
  # Day id for each intervention in this region
  end_lockdown_date <- ymd("2020-05-11")
  end_lockdown_id <- day_id_of(end_lockdown_date)
  covars <- covariates %>%
    # Convert to day id 
    mutate_all(day_id_of) %>%
    # Fill up : 0 before interv day id, 1 after (including day)
    apply(2, function(day_start){
      c(rep(0, day_start-1), rep(1, N2-day_start+1))
    }) %>%
    as.data.frame() %>% 
    mutate_at(vars(lockdown), function(indicator){
      if(lockdown_end){
        indicator[1:end_lockdown_id] %>% c(rep(0, N2 - end_lockdown_id))
      }else{
        indicator
      }
    }) %>%
    as.list()
  
  postlockdown <- c(
    rep(0, end_lockdown_id), 
    rep(1, N2 - end_lockdown_id)
  )
  
  # Add weekend indicator 
  is_weekend <- (lubridate::wday(region_first_day + 1:N2 - 1) %in% c(1,7)) %>%
    as.integer()
  holidays <- paste0("2020-", c("04-13", "05-01", "05-08", "05-21")) %>% 
    day_id_of() %>%
    subset( . <=length(is_weekend))
  is_weekend[holidays] <- 1
  
  # Add elections indicator
  elections_day_id <- day_id_of("2020-03-15")
  elections_indicator <- rep(0, N2)
  elections_indicator[elections_day_id] = 1
  
  # Region deaths vector
  deaths <- region_data %>% pull(deaths)
  
  # Return region-specific STAN input
  list(
    EpidemicStart=EpidemicStart,
    f = f,
    pop=pop,
    N=N,
    deaths_obs=deaths,
    mm = rep(keys$region_id, N),
    nn = region_data %>% pull(day_id),
    is_weekend = is_weekend,
    elections = elections_indicator,
    postlockdown = postlockdown,
    lockdown_end_id = day_id_of(end_lockdown_date)
  ) %>% 
    c(covars)
}

merge_regions_stan_inputs <- function(stan_data, region_stan_data, covariates){
  # bind covariates as matrix
  updated_covars <- 
    names(covariates) %>%
    map(function(i){
      cbind(stan_data[[i]], region_stan_data[[i]])
      
    }) %>%
    set_names(names(covariates))
  # merge 
  stan_data %>% 
    list_merge(!!!region_stan_data) %>%
    list_modify(
      f=cbind(stan_data$f, region_stan_data$f),
      is_weekend=cbind(stan_data$is_weekend, region_stan_data$is_weekend),
      elections=cbind(stan_data$elections, region_stan_data$elections),
      postlockdown=cbind(stan_data$postlockdown, region_stan_data$postlockdown),
      !!!updated_covars
    )
}
