suppressPackageStartupMessages({
  library(rstan)
  library(optparse)
  library(tidyverse)
  
  source("scripts/data_preproc/preproc_functions.R")
})

# ------------------------------------------------------------------------------

as_heatmap_dataset <- function(dataset){
  first_data_day <- dataset %>% 
      group_by(date) %>%
      tally() %>%
      filter(n == max(n)) %>%
      pull(date) %>% 
      min()
  sprintf("First day with data in all regions : %s", first_data_day) %>% message()
  dataset %>% 
    filter(date >= first_data_day) %>%
    ungroup() %>%
    complete(date=full_seq(.$date, 1), nesting(region, region_id))
  
}

missing_values_diagnostic <- function(dataset){
  
  na_count <- truncated_dataset %>% 
      filter(is.na(deaths)) %>%
      count(region, region_id)
  mean_missing <- na_count %>% pull(n) %>% mean()
  
  message("Missing values diagnostic : ")
  print(na_count)
  sprintf("Average missing days per region : %s", mean_missing) %>% message()
}

plot_heatmap <- function(hmap_dataset){
  invisible(Sys.setlocale(locale="en_US.utf-8"))
  hmap_dataset %>%
    filter(date )
    ggplot() +
    geom_tile(aes(x=region, y=date, fill=deaths)) +
    scale_x_discrete("Region") +
    scale_y_date(date_breaks = "2 weeks", labels= function(d){format(d, "%b %d")}) +
    scale_fill_viridis_c(trans="log1p", breaks=c(1,10,50,100, 500), labels=c(1,10,50,100, 500))+
    # scale_fill_continuous()+
    theme_bw() +
    theme(
      axis.text.x = element_text(angle=45, hjust=1),
      panel.grid.major.x = element_line(colour = "grey"),
      panel.border=element_blank())
}

datasets_diff <- function(d1, d2){
  d1 %>% left_join(d2, by=c("region", "region_id", "date")) %>%
    mutate(deaths = deaths.x - deaths.y)
}

# ------------------------------------------------------------------------------

merged_dataset <-
  read.csv("results/merge_spf_opencovid_agences_regionales_sante/mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset(keep.all=F) %>%
  as_heatmap_dataset()

opencovid_dataset <- 
  read.csv("results/opencovid_chiffres_cles//mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset(keep.all=F) %>%
  as_heatmap_dataset()

SPF_dataset <- 
  read.csv("results/SPF/regData.csv", sep=";") %>%
  filter(reg != "outre-mer") %>%
  transmute(
    region=str_replace_all(reg, '-', '.'), date=ymd(jour), deaths=dc, 
    region_id = as.factor(region) %>% as.integer())


datasets_diff(merged_dataset, SPF_dataset) %>% 
  plot_heatmap() +
  scale_fill_distiller(type="div", palette="RdYlGn", direction=1)

datasets_diff(merged_dataset, opencovid_dataset) %>% 
  plot_heatmap() +
  scale_fill_distiller(type="div", palette="RdYlGn", direction=1)

# heatmap_dataset <- as_heatmap_dataset(merged_dataset) 
# heatmap_dataset %>% missing_values_diagnostic()

(merged_heatmap <- plot_heatmap(merged_dataset))
(opencovid_heatmap <- plot_heatmap(opencovid_dataset))
(SPF_heatmap <- plot_heatmap(SPF_dataset))

opencovid_dataset %>%
  mutate(dataset="opencovid") %>%
  rbind(
    merged_dataset %>% mutate(dataset = "merged")
  ) %>%
  filter(region=="grand.est", date >= ymd("2020-03-01")) %>%
  ggplot(aes(x=date)) +
  geom_col(aes(y=deaths, fill=dataset), position="dodge")

cowplot::save_plot("report/figures/data_heatmap.png", merged_heatmap, base_height = 3, base_asp=1, scale=2)
      
