library(tidyverse)
library(lubridate)
library(ggpubr)
source("scripts/mixture_analysis/functions.R")

LOCKDOWN_END <- ymd("2020-05-11")

mixture_df <- data.table::fread(
  file="results/mixture_decoded_posterior.csv",
  colClasses=c(date="Date")
) %>%
  filter(date <= LOCKDOWN_END) %>%
  mutate(
    model = "mixture", 
    squared_err = (deaths - deaths_pred)^2
  ) %>%
  select(region, region_id, day_id, date, sample_id, deaths, squared_err, model)


loaded_data <- new.env()
load(
  "results/region-sparse-lockdown-latest.Rdata", 
  envir = loaded_data, verbose=T)
stanfit <- loaded_data$out
dataset <- loaded_data$grouped_dataset

default_df <- dataset %>% select(-t) %>%
  filter(date <= LOCKDOWN_END) %>%
  group_by(region, region_id, day_id, date, deaths) %>%
  group_modify(function(region_data, keys){
    mean_deaths <- stanfit$E_deaths[, keys$day_id, keys$region_id]
    phi <- stanfit$phi
    data.frame(
      predicted_deaths = rnbinom(phi, size=phi, mu=mean_deaths),
      sample_id = 1:length(phi)
    )
  }) %>%
  ungroup() %>%
  mutate(
    squared_err = (deaths - predicted_deaths)^2,
    model="base") %>%
  select(region, region_id, day_id, date, sample_id, deaths, squared_err, model)


final_df <-  
  rbind(default_df,mixture_df) %>%
  as_tibble() %>%
  group_by(region, region_id, sample_id, model) %>%
  summarise(squared_err=sum(squared_err, na.rm=T))

# Comparing error of models per region
(
  reg_err_plt <- final_df  %>%
    ggplot(aes(x=fct_rev(region))) +
    geom_boxplot(aes(y=squared_err, fill=model)) +
    scale_x_discrete("") +
    scale_y_log10("Squared error") +
    scale_fill_discrete("Model") +
    coord_flip() +
    theme_pubr() +
    theme(axis.text.x = element_text(angle=45, hjust=1))
)
cowplot::save_plot(
  "report/SupplementaryData/figures/error_compare_regions.png", 
  reg_err_plt,
  base_asp = 1, base_height = 7)

# Comparing total squared error
(
  err_comp_plt <- final_df %>%
    group_by(sample_id, model) %>%
    summarise(squared_err=sum(squared_err)) %>%
    ggplot(aes(x=model)) +
    geom_boxplot(aes(y=squared_err)) +
    scale_x_discrete("Model") +
    scale_y_continuous("Total squared error", breaks = seq(0, 2e6, 2e5)) +
    coord_flip() +
    theme_pubr()
)
cowplot::save_plot(
  "report/SupplementaryData/figures/error_compare_total.png", 
  err_comp_plt,
  base_asp=3, base_height=2)


