library(tidyverse)
library(rgdal)
library(broom)
library(maps)
library(colorspace)

if (!require(rgeos)) install.packages("rgeos")

pop_df <- read.csv(file = "data/french_region_pop.csv", h=F, col.names=c("id", "pop", "density"))
region_shapes <- rgdal::readOGR("data/shapefile", "regions-20180101") %>%
  broom::tidy(region="nom") %>%
  filter(long %>% between(-10, 10), lat >40) %>%
  inner_join(pop_df)

region_names <- region_shapes %>% 
  group_by(id) %>% 
  summarise_at(vars(lat, long), mean) %>%
  inner_join(pop_df)

metros <- read.csv(file="data/metropoles.csv")

base_map <- ggplot(region_shapes, aes(long, lat)) + cowplot::theme_map()

dir.create("report/SupplementaryData/figures/maps",showWarnings = F, rec=T)

(
  labelled_map <- base_map +
    geom_polygon(aes(group=group), col="grey60", fill=NA) + 
    geom_text(data=region_names, aes(long, lat, label=id), col="black")+
    geom_point(data=metros, aes(long, lat), 
      color="deepskyblue4", ) +
    geom_text(data=metros, aes(long, lat, label=nom, vjust=1.5 * (nom=="Marseille")), 
      hjust=-0.1, color="deepskyblue4", nudge_y = 0.1, fontface="bold") +
    coord_fixed(ratio=1.2)
)

cowplot::save_plot("report/SupplementaryData/figures/maps/geography.png", labelled_map, base_asp=1, scale=2)

(
  pop_map <- base_map + 
    geom_polygon(aes(group=group, fill=pop), col="grey60") + 
    geom_text(data=region_names, aes(col=(pop > 10000000),label=round(pop/1000000, 2)))+
    scale_color_manual(values=c("black", "white"), guide=F) +
    scale_fill_continuous_sequential("Heat", name="Population (M)", 
      labels=function(val) {sprintf("%d", trunc(val / 1000000))}) +
    coord_fixed(ratio=1.2) +
    theme(plot.margin = margin(t=0, b=0, unit="pt"))
)

cowplot::save_plot("report/SupplementaryData/figures/maps/population.png", pop_map, scale=1.5)

(
  density_map <- base_map + 
    geom_polygon(aes(group=group, fill=density), col="grey60") + 
    geom_text(data=region_names, aes(col=density > 900, label=density))+
    scale_color_manual(NULL, values=c("black", "white"), guide=F) +
    scale_fill_continuous_sequential("Heat", name=expression(
      paste("Pop. per ", km^2))) +
    coord_fixed(ratio=1.2) +
    theme(plot.margin = margin(t=0, b=0, unit="pt"))
)
cowplot::save_plot("report/SupplementaryData/figures/maps/density.png", density_map, scale=1.5)

