library(tidyverse)
library(rgdal)
library(broom)
library(maps)
library(colorspace)
library(lubridate)
source("scripts/plots/utils/utils.R")

if (!require(rgeos)) install.packages("rgeos")

LOCKDOWN <- ymd("2020-03-17")
LOCKDOWN_END <- ymd("2020-05-11")
mixture_df <- data.table::fread(file="results/mixture_decoded_posterior.csv") %>%
  mutate(date = ymd(date)) %>%
  filter(date <= LOCKDOWN_END) %>%
  as_tibble()


(
  Rt_df <- mixture_df %>% 
    filter(date == LOCKDOWN | day_id == 1) %>%
    group_by(region_id, date, day_id) %>%
    summarise(Rt = median(Rt)) %>%
    mutate(variable_name=if_else(day_id==1, "R0", "lockdown"))
)

region_shapes <- rgdal::readOGR("data/shapefile", "regions-20180101") %>%
  broom::tidy(region="nom") %>%
  filter(long %>% between(-10, 10), lat >40) %>%
  mutate(region=id, region_id=as.integer(as.factor(region))) %>%
  inner_join(Rt_df, by="region_id")

region_names <- region_shapes %>% 
  group_by(region_id, region) %>% 
  summarise_at(vars(lat, long), mean) %>%
  inner_join(Rt_df, by="region_id")

plot_Rt <- function(name){
  base_map <- region_shapes %>% 
    filter(variable_name == name) %>%
    ggplot(aes(long, lat)) + cowplot::theme_map()
  
  base_map + 
    geom_polygon(aes(group=group, fill=Rt), col="grey60") + 
    geom_text(
      data=region_names %>% filter(variable_name == name), 
      aes(col=Rt > 3, label=round(Rt, 2))
    )+
    scale_color_manual(NULL, values=c("black", "white"), guide=F) +
    scale_fill_continuous_sequential("Heat") +
    coord_fixed(ratio=1.2)
}

(R0_map <- plot_Rt("R0") + labs(fill=expression(R[0])))
(Rt_lockdown_map <- plot_Rt("lockdown") + labs(fill=expression(R['lockdown'])))

dir.create("report/SupplementaryData/figures/maps", showWarnings = F, rec=T)
cowplot::save_plot(
  "report/SupplementaryData/figures/maps/R0.png", 
  R0_map, scale=1.5)
cowplot::save_plot(
  "report/SupplementaryData/figures/maps/Rlockdown.png", 
  Rt_lockdown_map, scale=1.5)
