suppressPackageStartupMessages({
  library(tidyverse)
  library(bayesplot)
  library(cowplot)
  library(ggpubr)
  library(EnvStats)
  library(optparse)
  source("scripts/plots/utils/utils.R")
  invisible(Sys.setlocale(locale="en_US.utf-8"))
  
})
# ------------------------------------------------------------------------------
# Commandline options and parsing
parser <- OptionParser() %>%
  add_option(c("-m", "--mode"),
    action="store",
    default="lockdown",
    help="Plot Rt for model : [lockdown | elections | weekends") %>%
  add_option(c("-o", "--output"),
    action="store",
    help="Figure output file path") 
# %>%
#   add_option(c("-n", "--ndays"),
#     action="store",
#     default=105,
#     help="Number of days")

cmdoptions <- parse_args(parser,
  args = commandArgs(trailingOnly = TRUE),
  positional_arguments = TRUE)


MODE <- cmdoptions$options$mode
stopifnot(MODE %in% c("lockdown", "elections", "weekends"))

filename <- cmdoptions$args[1]

# filename <- "results/region-sparse-lockdown-full.Rdata"
# filename <- "results/region-sparse-elections-paper.Rdata"

# load variables : grouped_dataset, out
sprintf("LOADING : %s",filename) %>% message()

loaded_data <- new.env()
load(filename, verbose=T, envir = loaded_data)
stan_data <- loaded_data$stan_data
stanfit <- loaded_data$out
dataset <- loaded_data$grouped_dataset


simulate_Rt_prior <- function(stan_data, region_id, n=10000){
  
  generate_alpha_indic <- function(onehot){
    alpha <- rgammaAlt(n, 1/6, 1) - log(1.05)/6.0
    alpha %*% t(onehot)
  }
  
  N0 <- stan_data$N0
  N2 <- stan_data$N2
  SI <- pgammaAlt(1:N2, 6.5, 0.62) - pgammaAlt((1:N2)-1, 6.5, 0.62)
  
  R0_sd <- abs(rnorm(n, 0, 0.5))
  R0 <- rnorm(n, 3.28, R0_sd)
  interv_effects <-  (- generate_alpha_indic(stan_data$lockdown[,region_id]))
  if (MODE == "elections"){
    interv_effects = interv_effects + generate_alpha_indic(stan_data$elections[,region_id])
  } else if (MODE == "weekends"){
    interv_effects = interv_effects - generate_alpha_indic(stan_data$is_weekend[,region_id])
  }
  Rt_prior <- R0 * exp(interv_effects)
  Rt_prior_adj <- Rt_prior[,1:N0]
  
  tau <- rexp(n, 0.03)
  initial_cases <- rexp(n, 1/tau)
  new_cases <- matrix(rep(initial_cases, N0), ncol=N0)
  pop <- stan_data$pop[region_id]
  
  init_args=list(
    new_cases=new_cases,
    Rt_prior_adj=Rt_prior_adj
  )
  reduce((N0+1):N2, .init=init_args, 
    .f=function(acc, day_id){
      prev_days <- 1:(day_id-1)
      convolution <- acc$new_cases[,prev_days] %*% SI[day_id-prev_days]
      cases_cumsum <- rowSums(acc$new_cases[,prev_days])
      Rt_prior_adj <- ((pop - cases_cumsum) / pop) * Rt_prior[,day_id]
      new_cases <- convolution * Rt_prior_adj
      list(
        Rt_prior_adj = acc$Rt_prior_adj %>% cbind(Rt_prior_adj),
        new_cases = acc$new_cases %>% cbind(new_cases)
      )
    }) %>% use_series(Rt_prior_adj)
}

# ------------------------------------------------------------------------------

Rt_prior <- dataset %>% 
  group_map(stan_data, stanfit,
    .f=function(region_data, keys, stan_data, stanfit, n=10000){
      message(keys$region)
      region_id <- keys$region_id
      Rt_prior_quantiles <- 
        simulate_Rt_prior(stan_data, region_id) %>%
        extract_mcmc_quantiles("Rt_prior")
      Rt_posterior_quantiles <- extract_mcmc_quantiles(stanfit$Rt_adj[,,region_id], "Rt_post")
      
      rbind(Rt_prior_quantiles, Rt_posterior_quantiles) %>%
        mutate(
          region = keys$region, 
          region_id = keys$region_id,
          date = min(region_data$date) + day_id - 1)
    }) %>%
  bind_rows()

(ribbon_plot <- Rt_prior %>% 
    filter(region_id == 8) %>%
    ggplot(aes(x=date)) +
    geom_ribbon(aes(ymin=min2, ymax=max2, fill=variable_name, group=rev(variable_name))) +
    geom_ribbon(aes(ymin=min, ymax=max, color=variable_name), alpha=0, lty="dashed") +
    scale_x_date("Date", breaks="2 weeks", date_labels="%b %d")+
    scale_y_continuous("Rt", breaks=1:6) +
    scale_fill_manual("50% CI", labels=c("Posterior", "Prior"), 
      values=alpha(c("deepskyblue4", "orangered"), c(0.8, 0.7))) +
    scale_color_manual("95% CI", labels=c("Posterior", "Prior"),
      values=alpha(c("deepskyblue4","orangered"), c(0.8,0.7))) +
    theme_pubr()
)
output_path <- cmdoptions$options$output
sprintf("Output figure to %s", output_path) %>% message()
dir.create(dirname(output_path), rec=T, showWarnings = F)
cowplot::save_plot(output_path, plot = ribbon_plot, base_asp=2)
