suppressPackageStartupMessages({
  library(tidyverse)
  library(bayesplot)
  library(cowplot)
  library(ggpubr)
  library(EnvStats)
  library(optparse)
})
# ------------------------------------------------------------------------------
# Commandline options and parsing
parser <- OptionParser() %>%
  add_option(c("-a", "--alpha"),
    action="store",
    help="Index of alpha to plot") %>%
  add_option(c("-o", "--output"),
    action="store",
    help="Figure output file path") %>%
  add_option(c("-l", "--xlim"),
    action="store",
    type="double",
    default=NA,
    help="Upper bound for X axis")
  
  
cmdoptions <- parse_args(parser,
  args = commandArgs(trailingOnly = TRUE),
  positional_arguments = TRUE)

filename <- cmdoptions$args[1]
ALPHA_INDEX <- cmdoptions$options$alpha
OUTPUT_PATH <- cmdoptions$options$output
x_upper_bound <- cmdoptions$options$xlim

# load variables : grouped_dataset, out
sprintf("LOADING : %s",filename) %>% message()

loaded_data <- new.env()
load(filename, verbose=T, envir = loaded_data)
stanfit <- loaded_data$out

if(is.null(ALPHA_INDEX)){
  alpha <- stanfit$alpha
} else {
  alpha <- stanfit$alpha[,as.integer(ALPHA_INDEX)]
}

# Histogram
prior <- data.frame(x=seq(0, ceiling(max(alpha)), 0.01)) %>%
  mutate(
    gamma_density = dgammaAlt(x, .1667, 1),
    x_shifted= x - ( log(1.05) / 6.0 ))

(
  plothist <- data.frame(alpha=alpha) %>%
    ggplot() + 
    geom_histogram(aes(x=alpha, y=..density..), 
      col="black", fill="white", binwidth=0.03, alpha=0.5) +
    geom_line(data=prior, aes(x=x_shifted, y=gamma_density, col="red")) +
    xlim(c(0, x_upper_bound)) +
    scale_color_manual(name="", values="red", label="Prior") +
    scale_linetype_manual(name="", values="solid", label="Prior") +
    theme_pubr()
)

# QQ-plot
qgammaShifted <- function(p, mean, cv){
  qgammaAlt(p, mean, cv) -  ( log(1.05) / 6.0 )
}

(
  plotqq <- data.frame(alpha=alpha) %>%
    ggplot(aes(sample=alpha)) +
    geom_qq(distribution=qgammaShifted, dparams=c(1/6, 1), alpha=0.2) +
    geom_qq_line(distribution=qgammaAlt, dparams=c(1/6, 1), col="red") +
    scale_x_continuous("Quantiles of prior") +
    scale_y_continuous("Quantiles of samples") +
    theme_pubr()
)

# Layout in 2 panels
(fig <- cowplot::plot_grid(plotqq, plothist, align="hv", axis="tblr", nrow=1))

sprintf("Output figure to %s", OUTPUT_PATH) %>% message()
dir.create(dirname(OUTPUT_PATH), rec=T, showWarnings = F)
cowplot::save_plot(OUTPUT_PATH, plot = fig, base_asp = 2)

