suppressPackageStartupMessages({
  library(tidyverse)
  library(ggpubr)
  library(lubridate)
  invisible(Sys.setlocale("LC_ALL", locale="en_US.utf8"))
  source("scripts/data_preproc/preproc_functions.R")
})


LOCKDOWN <- ymd("2020-03-17")
LOCKDOWN_END <- ymd("2020-05-11")

mixture <- data.table::fread(
  file="results/simulated_datasets/mixture_lockdown_shift_decoded_posterior.csv",
  colClasses = c(date="Date", lockdown_day="Date")
) %>%
  mutate(
    model="mixture",
    sim_id = dense_rank(sample_id)
  ) %>%
  select(
    region, region_id, sim_id, sample_id, day_id, date, 
    deaths_mean, deaths_pred, Rt, new_cases, lockdown_day, model
  ) %>%
  as_tibble()

q <- c(0.025, 0.25, 0.5, 0.75,  0.975) %>% set_names(paste0(.*100, "%"))
q_funs <- map(q, ~partial(quantile, probs=.x, na.rm=TRUE))
mixture %>% group_by(lockdown_day, sample_id) %>%
  summarise(total_deaths=sum(na.omit(deaths_pred))) %>%
  group_by(lockdown_day) %>%
  summarise_at(vars(total_deaths), q_funs) %>%
  write_csv(path="results/lockdown_shift_total_deaths.csv")

grouped_dataset <- 
  read.csv("results/merge_spf_opencovid_agences_regionales_sante/mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset() %>%
  group_by(region, region_id) %>%
  group_modify(function (region_data, keys) {
    message(keys$region %>% str_to_upper())
    message(" + PADDING : ")
    region_data <- pad_region_data(region_data)
    message(" + SLICING : ")
    region_data <- slice_from_onset(region_data)
  })

obs_total_deaths <- sum(grouped_dataset$deaths)

(
  report_plt <- mixture %>%
    group_by(lockdown_day, sim_id) %>%
    summarise(total_deaths=sum(na.omit(deaths_pred))) %>% 
    mutate(actual = if_else(lockdown_day == LOCKDOWN, "actual", "counterfactual")) %>%
    ggplot(aes(x=lockdown_day)) +
    geom_hline(
      aes(yintercept=obs_total_deaths, lty="observed"),
      col="orangered", lwd=0.4) +
    geom_boxplot(
      aes( y=total_deaths, group=lockdown_day, fill=actual), 
      lwd=0.3) +
    scale_x_date("Lockdown start", date_breaks="2 days", date_labels="%b %d") +
    scale_y_continuous("Total mortality") +
    scale_fill_discrete("") +
    scale_linetype_manual(NULL, values=c("dashed")) +
    theme_pubr()
)

cowplot::save_plot("report/figures/lockdown_shift_mixture.png", report_plt)
