library(ggpubr)
library(cowplot)
library(scales)
library(lubridate)

source("scripts/plots/utils/geom-stepribbon.r")
source("scripts/plots/utils/utils.R")

LOCKDOWN <- ymd("2020-03-17")

plot_cases <- function(region_data, region) {
  region_data %>% filter(variable_name == "predicted_cases") %>%
    ggplot(aes(x = date)) +
    geom_ribbon(aes(
      ymin = min,
      ymax = max,
      fill = "95%"
    )) +
    geom_ribbon(aes(
      ymin = min2,
      ymax = max2,
      fill = "50%"
    )) +
    geom_vline(xintercept = LOCKDOWN, lty = "dashed") +
    ggtitle(region) +
    ylab("Daily number of infections") +
    scale_x_date(date_breaks = "2 weeks", labels = date_format("%e %b")) +
    scale_y_continuous(expand = c(0, 0), labels = comma) +
    scale_fill_manual(
      name = "",
      labels = c("50%", "95%"),
      values = c(alpha("deepskyblue4", 0.55), alpha("deepskyblue4", 0.45))
    ) +
    theme_pubr(base_family = "sans") +
    theme(
      axis.title.x = element_blank(),
      axis.text.x = element_text(angle = 45, hjust = 1),
      legend.position = "None"
    ) +
    guides(fill = guide_legend(ncol = 1))
}


plot_deaths <- function(region_data, lockdown.legend = F, plot.lockdown=T) {
  mean_deaths_df <-
    region_data %>% filter(variable_name == "mean_deaths")
  predicted_deaths_df <-
    region_data %>% filter(variable_name == "predicted_deaths")
  
  plt <- ggplot(data = region_data, aes(x = date)) +
    geom_col(
      aes(y = deaths, fill = "reported"),
      fill = "coral4",
      stat = 'identity',
      position = "identity",
      alpha = 0.15
    ) +
    geom_ribbon(data = mean_deaths_df,
      aes(
        x = date,
        ymin = min,
        ymax = max,
        fill = "95%"
      )) +
    geom_ribbon(data = mean_deaths_df,
      aes(
        x = date,
        ymin = min2,
        ymax = max2,
        fill = "50%"
      )) +
    geom_ribbon(
      data = predicted_deaths_df,
      aes(
        x = date,
        ymin = min,
        ymax = max,
        col = variable_name
      ),
      alpha = 0.0,
      lty = "dotted"
    ) +
    scale_x_date(date_breaks = "2 weeks", labels = date_format("%e %b")) +
    ylab("Daily number of deaths") +
    scale_color_manual(
      name = NULL,
      values = "deepskyblue4",
      labels = "Mortality 95% CI") +
    scale_fill_manual(
      name = NULL,
      labels = sprintf("Average %s CI ", c("50%", "95%")),
      #"Mean deaths",
      values = c(alpha("deepskyblue4", 0.7), alpha("deepskyblue4", 0.45))
    ) +
    theme_pubr(base_family = "sans") +
    theme(
      axis.title.x = element_blank(),
      axis.text.x = element_text(angle = 45, hjust = 1),
      legend.position = "right"
    ) +
    guides(fill = guide_legend(ncol = 1))
  
  if (lockdown.legend) {
    plt +
      geom_vline(aes(xintercept = LOCKDOWN, lty = "Lockdown")) +
      scale_linetype_manual("", values = "dashed")
  } else if(plot.lockdown) {
    plt + geom_vline(xintercept = LOCKDOWN, lty = "dashed")
  } else {
    plt
  }
}


plot_rt <- function(region_data, region, covariates) {
  plot_data <- region_data %>% filter(variable_name == "rt")
  
  marker_y_position <- plot_data %>% pull(max) %>% max()
  region_covars <- covariates %>%
    group_by(date) %>%
    group_modify(function(same_date_records, date) {
      same_date_records %>% mutate(y = marker_y_position)
    }) %>%
    filter(interv == "lockdown")
  
  plot_data %>%
    ggplot(aes(x = date)) +
    geom_stepribbon(aes(
      ymin = min,
      ymax = max,
      fill = "95%"
    )) +
    geom_stepribbon(aes(
      ymin = min2,
      ymax = max2,
      fill = "50%"
    )) +
    geom_hline(yintercept = 1,
      color = 'black',
      size = 0.1) +
    geom_vline(aes(xintercept = LOCKDOWN, lty = "lockdown")) +
    ylab(expression(R[t])) +
    scale_fill_manual(
      name = NULL,
      labels = c("50% CI", "95% CI"),
      values = c(alpha("seagreen", 0.75), alpha("seagreen", 0.5))
    ) +
    scale_linetype_manual("", values = "dashed") +
    scale_x_date(NULL, date_breaks = "2 weeks", labels = date_format("%e %b")) +
    scale_y_continuous(expand = expansion(mult = c(0, 0.1))) +
    expand_limits(y = 0) +
    theme_pubr(base_family = "sans") +
    theme(axis.text.x = element_text(angle = 45, hjust = 1),
      legend.position = "right")
}



save_plots <- function(region, cases, deaths, rt, filename) {
  # Make 3 panel
  three_panel_plot <- plot_grid(cases,
    deaths,
    rt,
    ncol = 1,
    align = "vh",
    axis = "lrtb")
  
  # Save 3p plot
  out_label <- extract_file_label(filename)
  out_dir <- file.path("figures", out_label, "three_panels")
  dir.create(out_dir, showWarnings = F, recursive = T)
  
  region_label <- str_replace_all(region, '\\.', '-')
  out_filename <-
    file.path(out_dir, sprintf("%s.png", region_label))
  sprintf(" | Saving plot @ %s", out_filename) %>% message()
  save_plot(
    filename = out_filename,
    three_panel_plot,
    base_aspect_ratio = 2.3 / 3,
    base_height = 10
  )
}
