extract_file_label <- function(filename){
  filename %>% basename() %>% str_remove("\\.Rdata$")
}


make_out_filename <- function(filename, plot_label){
  file_label <- extract_file_label(filename)
  out_dir <- file.path("figures", file_label)
  dir.create(out_dir, showWarnings = F, recursive = T)
  
  file.path(out_dir, plot_label)
}

# returns df with columns : 
# - day_id, variable_name, mean, 
# - [quantile for each p in probs]
# - [cumsum columns of mean and quantile]
extract_mcmc_quantiles <- 
  function(
    mcmc_data, variable_name, 
    probs=c(.025, .975, .25, .75, .5),
    quantile_names=c("min", "max", "min2", "max2", "median")
  ){
    mcmc_data %>% unname() %>%
      matrixStats::colQuantiles(probs=probs)%>%
      as.data.frame() %>%
      set_names(quantile_names) %>% 
      rownames_to_column("day_id") %>%
      mutate(
        day_id=as.integer(day_id),
        mean=colMeans(mcmc_data),
        variable_name = variable_name) 
  }


add_sample_deaths_negbinom <- function(mcmc_out){
  phi <- mcmc_out$phi
  res <- apply(mcmc_out$E_deaths, c(2,3), FUN = function(deaths_mean_samples){
    rnbinom(phi, size=phi, mu=deaths_mean_samples)
  })
  mcmc_out %>% list_modify(predicted_deaths = res)
}

make_plot_data_region <- function(region_data, keys, mcmc_out, ...){
  # using last day id to know the length of "observed" period
  N <- region_data %>% pull(day_id) %>% max() 
  region <- keys$region
  region_id <- keys$region_id
  
  mcmc_out <- add_sample_deaths_negbinom(mcmc_out)
  
  observed_data <- region_data %>% select(date, deaths) %>% mutate(obs_deaths_csum=cumsum(deaths))
  
  map2_dfr(
    c("prediction", "E_deaths", "Rt_adj", "predicted_deaths"), 
    c("predicted_cases", "mean_deaths", "rt", "predicted_deaths"),
    function(key, name_prefix){
      mcmc_data <- mcmc_out[[key]][,1:N,region_id]
      quants <- extract_mcmc_quantiles(mcmc_data, name_prefix, ...)  
      csum_prefix <- paste(name_prefix, "csum", sep="_")
      csum_quants <- mcmc_data %>% 
        # cumsum over days
        apply(1, cumsum) %>% 
        t() %>% 
        extract_mcmc_quantiles(csum_prefix, ...)
      
      rbind(quants, csum_quants)
    })  %>%
    mutate(
      date=min(region_data$date) + day_id -1,
      region=region,
      region_id=region_id
    ) %>%
    left_join(observed_data, by="date") %>%
    fill(obs_deaths_csum, .direction = "down")
}
