suppressPackageStartupMessages({
  library(rstan)
  library(optparse)
  library(tidyverse)
  library(EnvStats) # required for pgammaAlt()

  source("scripts/data_preproc/preproc_functions.R")
  source("scripts/data_preproc/stan-input-functions.R")
})

# ------------------------------------------------------------------------------
# Commandline options and parsing
parser <- OptionParser() %>%
  add_option(c("-D", "--debug"),
    action="store_true",
    default=F,
    help="Perform a debug run of the model") %>%
  add_option(c("-F", "--full"),
    action="store_true",
    default=F,
    help="Perform a full run of the model") %>%
  add_option(c("-i", "--id"),
    action="store",
    help="Set run ID") %>%
  add_option(c("-p", "--prepare"),
    action="store_true",
    default=F,
    help="Only prepare dataset without running the model") %>%
  add_option(c('-l', '--limit'),
    action="store",
    type="integer",
    help="Maximum number of days to observe") %>%
  add_option(c('-s', '--skip'),
    action="store",
    type="integer",
    help="Skip N last days in data") %>%
  add_option(c('-u', '--upper'),
    action="store_true",
    default=F,
    help="Lower or upper priors for the alpha mixture") %>%
  add_option(c('-r', '--ifr-scale'),
    action="store",
    dest="ifr_scale",
    type="double",
    default=1.0,
    help="Multiplicative factor for IFR") %>%
  add_option(c("-c", "--complete-data"),
    action="store_true",
    dest="complete_data",
    default=F,
    help="Use complete dataset instead of truncating at May 11."
  )


cmdoptions <- parse_args(parser,
  args = commandArgs(trailingOnly = TRUE),
  positional_arguments = TRUE)

DEBUG = cmdoptions$options$debug
FULL = cmdoptions$options$full
if(DEBUG && FULL) {
  stop("Setting both debug and full run modes at once is invalid")
}

STAN_MODEL = cmdoptions$args[1]
MODEL_NAME = basename(STAN_MODEL) %>% str_remove("\\.stan")

JOBID = cmdoptions$options$id
if(is.null(JOBID)){
  if(DEBUG) {
    message("Running in DEBUG mode")
    JOBID = "debug"
  } else if (FULL) {
    message("Running in FULL mode")
    JOBID= "full"
  }
}

LIMIT <- cmdoptions$options$limit
SKIP_LAST_N <- cmdoptions$options$skip
UPPER <- cmdoptions$options$upper
IFR_SCALE <- cmdoptions$options$ifr_scale
POST_LOCKDOWN <- cmdoptions$options$complete_data

## Ensure that output directories exist
c("results/", "figures/", "web/") %>%  walk(dir.create, showWarnings=F, recursive=TRUE)

# ------------------------------------------------------------------------------

filename <- paste0(MODEL_NAME, '-', JOBID)
stanfit_output <- file.path('results/', sprintf("%s.Rdata", filename))
message(sprintf("MODEL : %s", MODEL_NAME))
message(sprintf("Job ID : %s", JOBID))
message(sprintf("Filename prefix : %s", filename))
message(sprintf("STAN output @ %s \n\n", stanfit_output))

# Sys.sleep(3)

# ------------------------------------------------------------------------------
# STAN dataset preparation

fr_dataset <-
  read.csv("results/merge_spf_opencovid_agences_regionales_sante/mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset(post.lockdown=POST_LOCKDOWN)

regions <- unique(fr_dataset$region) %>% sort()

covariates <- load_covariates("data/interventions.csv")

## Load french region population data
ifr_pop_data <-
  read.csv("data/region_wise_IFR.csv", sep=";") %>%
  prepare_regions_pop_ifr(IFR_SCALE)

grouped_dataset <- fr_dataset %>%
  group_by(region, region_id) %>%
  group_modify(function (region_data, keys) {
    message(keys$region %>% str_to_upper())
    message(" + PADDING : ")
    region_data <- pad_region_data(region_data)
    message(" + SLICING : ")
    region_data <- slice_from_onset(region_data)
    return(region_data)
  })

N2 = as.integer(max(grouped_dataset$date) - min(grouped_dataset$date)) + 1

if( !is.null(LIMIT) ){
  grouped_dataset <- grouped_dataset %>%
    filter( day_id <=LIMIT )
}
if( !is.null(SKIP_LAST_N) ){
  grouped_dataset <- grouped_dataset %>%
    filter( date <= max(.$date) - SKIP_LAST_N )
}

last_data_day <- max(grouped_dataset$date)
sprintf("\n*** Last day observed : %s\n", last_data_day) %>% message()

# total number of days (observed + missing + forecast)


# Next step : convert to STAN input dataset

stan_data <- grouped_dataset %>%
  group_map(make_region_stan_input, covariates, ifr_pop_data, N2) %>%
  # Merge region-specific data together
  reduce(merge_regions_stan_inputs, covariates) %>%
  # Add non-region-specific STAN inputs
  list_modify(
    K = 2, # Number of mixture components
    N_obs = sum(.$N), # number of days with observed data
    M = length(regions),
    N2 = N2, # total number of days (observed + missing + forecast)
    N0 = 6,  # N0 = 6 to make it consistent with Rayleigh
    # contamination time Gamma distribution
    SI = pgammaAlt(1:N2, 6.5, 0.62) - pgammaAlt((1:N2)-1, 6.5, 0.62),
    last_day_observed = last_data_day,
    regions = regions
  )

if(stan_data$N2 < max(stan_data$nn)){
  sprintf(
    "N2 = %d is insufficient to cover all %2$d observed days. Increase N2 to at least %2$d",
    stan_data$N2, max(stan_data$nn)
    ) %>%
    warning()
  stop("Invalid parameter")
}


message("+--------------------------------------------------------------------")
message("| STAN DATA READY : ")
message("+--------------------------------------------------------------------")

stan_data %>% str() %>% message()

# ------------------------------------------------------------------------------
# MCMC sampling
# ------------------------------------------------------------------------------

if( !cmdoptions$options$prepare ) {

  options(mc.cores = parallel::detectCores())
  rstan_options(auto_write = TRUE)

  # Compile STAN model
  model <- stan_model(STAN_MODEL)

  # Start MCMC sampling
  if(DEBUG) {
    fit <- sampling(model, data=stan_data, iter=40, warmup=20, chains=2)
        # init = function(){list(alpha=c(0.0, 2)) } )
  } else if (FULL) {
    print("USING OPTIONS FOR FULL RUN")
    fit <- sampling(
      model, data=stan_data,
      iter=4000, warmup=2000, chains=4, thin=4,
      control = list(adapt_delta = 0.95, max_treedepth = 15))
          # init = function(){list(alpha=c(0.0, 2)) })
  } else {
    fit <- sampling(
      model, data=stan_data,
      iter=200, warmup=100, chains=4, thin=4,
      control = list(adapt_delta = 0.95, max_treedepth = 10))
          # init = function(){list(alpha=c(0.0, 2)) })
  }

  message("+--------------------------------------------------------------------")
  message("| SAMPLING DONE ")
  message("+--------------------------------------------------------------------")

  out <- rstan::extract(fit)

  # Save dataset and sampled results
  message(sprintf("Saving STAN fit data @ %s", stanfit_output))
  save(fit, out, grouped_dataset, covariates, stan_data, file=stanfit_output)

}
