suppressPackageStartupMessages({
  library(rstan)
  library(optparse)
  library(tidyverse)
  library(pbapply)
  library(ggpubr)
  library(lubridate)
  
  source("scripts/data_preproc/preproc_functions.R")
  source("scripts/data_preproc/stan-input-functions.R")
})

# Commandline options and parsing
parser <- OptionParser() %>%
  add_option(c("-n", "--ndays"),
    action="store",
    default = NULL,
    type="integer",
    help="Number of days N2") %>%
  add_option(c("-s", "--nsim"),
    action="store",
    default = 500,
    help="Number of simulations to run") %>%
  add_option(c("-o", "--output"),
    action="store",
    default="results/elections-effect-size-simulation.Rdata",
    help="Simulation output destination")

cmdoptions <- parse_args(parser,
  args = commandArgs(trailingOnly = TRUE),
  positional_arguments = TRUE)

N2 <- cmdoptions$options$ndays
N_SIM <- as.integer(cmdoptions$options$nsim)
OUTPUT_FILENAME <- cmdoptions$options$output

# ------------------------------------------------------------------------------
# STAN dataset preparation

fr_dataset <-
  read.csv("results/merge_spf_opencovid_agences_regionales_sante/mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset()

regions <- unique(fr_dataset$region) %>% sort()

covariates <- load_covariates("data/interventions.csv")

## Load french region population data
ifr_pop_data <-
  read.csv("data/region_wise_IFR.csv", sep=";") %>%
  prepare_regions_pop_ifr()

grouped_dataset <- fr_dataset %>%
  group_by(region, region_id) %>%
  group_modify(function (region_data, keys) {
    message(keys$region %>% str_to_upper())
    message(" + PADDING : ")
    region_data <- pad_region_data(region_data)
    message(" + SLICING : ")
    region_data <- slice_from_onset(region_data)
  })

# total number of days (observed + missing + forecast)
if(is.null(N2)){
  N2 <- as.integer(max(grouped_dataset$date) - min(grouped_dataset$date)) + 1 
} 

# Next step : convert to STAN input dataset
stan_data_initial <- grouped_dataset %>% 
  group_map(make_region_stan_input, covariates, ifr_pop_data, N2) %>%
  # Merge region-specific data together
  reduce(merge_regions_stan_inputs, covariates) %>%
  # Add non-region-specific STAN inputs
  list_modify(
    K = 2, # Number of mixture components
    N_obs = sum(.$N), # number of days with observed data
    M = length(regions),
    N2 = N2, # total number of days (observed + missing + forecast)
    N0 = 6,  # N0 = 6 to make it consistent with Rayleigh
    SI = pgammaAlt(1:N2, 6.5, 0.62) - pgammaAlt((1:N2)-1, 6.5, 0.62),
    regions = regions
  )

str(stan_data_initial) %>% print()

# ------------------------------------------------------------------------------

source("scripts/simulations/sim_functions.R")

options(mc.cores = parallel::detectCores())
pboptions(type="txt") 
rstan_options(auto_write = TRUE)

message("Compiling model")
model_sim <- stan_model('models/simulations/simulation-election.stan')

# Load reference fit for parameters calibration
fitted_data <- new.env()
load("results/region-sparse-lockdown-latest.Rdata", fitted_data, verbose = T)
stanfit <- fitted_data$out

# Simulations
sprintf("Running %s simulations on days N2 = %s", N_SIM, N2) %>% message()

Rt_modifiers <- c(1, 1.25, 1.5, 2)
samples <- sample(1:N_SIM)

simulated_dataset <- pblapply(
  Rt_modifiers, samples, grouped_dataset, stan_data_initial,
  cl = parallel::detectCores() - 1,
  FUN=function(Rt_mod, samples, dataset, stan_data_initial){ 
    data_modifier <- make_Rt_stan_modifier(stan_data_initial, stanfit, Rt_mod)
    lapply(samples, simulation, model_sim, data_modifier) %>%
      simulation_post_process(dataset) %>%
      mutate(Rt_mod=Rt_mod)
  }) %>%
  bind_rows()

save(grouped_dataset, simulated_dataset, N2, N_SIM, file=OUTPUT_FILENAME)

