shift_lockdown <- function(stan_data, n){
  fill <- as.integer(n<0)
  stan_data %>% 
    list_modify(
      lockdown_day = LOCKDOWN + n,
      lockdown = 
        stan_data$lockdown %>% 
        apply(2, data.table::shift, n=n, fill=fill, type="lag")
    )
}

make_default_stan_modifier <- function(stan_data_initial, stanfit){
  function(sample_id){
    stan_data_initial %>%
      list_modify(
        ifr_noise = stanfit$ifr_noise[sample_id,],
        mu = stanfit$mu[sample_id,],
        alpha = stanfit$alpha[sample_id],
        y = stanfit$y[sample_id,],
        phi = stanfit$phi[sample_id]
      )
  }
}


make_Rt_stan_modifier <- function(stan_data_initial, stanfit, Rt_mod){
  function(sample_id){
    stan_data_initial %>%
      list_modify(
        ifr_noise = stanfit$ifr_noise[sample_id,],
        mu = stanfit$mu[sample_id,],
        alpha = stanfit$alpha[sample_id],
        y = stanfit$y[sample_id,],
        phi = stanfit$phi[sample_id],
        Rt_mod = Rt_mod
      )
  }
}


make_mixture_stan_modifier <- function(stan_data_initial, stanfit, posterior_dataset=NULL){
  if (is.null(posterior_dataset)){
    function(sample_id){
      stan_data_initial %>%
        list_modify(
          ifr_noise = stanfit$ifr_noise[sample_id,],
          mu = stanfit$mu[sample_id,],
          alpha = stanfit$alpha[sample_id,],
          y = stanfit$y[sample_id,],
          phi = stanfit$phi[sample_id],
          theta = stanfit$theta[sample_id, ]
        )
    }
  } else {
    posterior.samples <- posterior_dataset %>%
      group_by(theta, sample_id) %>%
      summarise_at(vars(post), sum) %>%
      pivot_wider(values_from = post, names_from = theta) %>%
      select(-sample_id) %>%
      as.matrix()
    
    stanfit.weighted <- stanfit %>%
      list_modify(
        alpha = rowSums(stanfit$alpha * posterior.samples) / rowSums(posterior.samples)
      )
    make_default_stan_modifier(stan_data_initial, stanfit.weighted)
  }
}

extract_sample <- function(mcmc_sample, name, regions){
  mcmc_sample %>% 
    drop() %>% 
    as.data.frame() %>% 
    magrittr::set_names(regions) %>%
    rownames_to_column("day_id") %>%
    pivot_longer(cols=-day_id, names_to="region", values_to=name) %>%
    mutate(day_id=as.integer(day_id))
}

default_extractor <- function(mcmc_out, stan_data){
  regions <- stan_data$regions
  
  deaths_df <- extract_sample(mcmc_out$deaths, "deaths_pred", regions) 
  mean_deaths_df <- extract_sample(mcmc_out$E_deaths, "deaths_mean", regions) 
  preds_df <- extract_sample(mcmc_out$prediction, "new_cases", regions) 
  Rt_df <- extract_sample(mcmc_out$Rt_adj, "Rt", regions)
  
  keys <- c("region", "day_id")
  deaths_df %>%
    inner_join(mean_deaths_df, by=keys) %>%
    inner_join(preds_df, by=keys) %>%
    inner_join(Rt_df, by=keys)  
}

simulation <- function(sample_id, model, stan_data_modifier, extractor=default_extractor){
  
  stan_data <- stan_data_modifier(sample_id)
  fit <- sampling(model,data=stan_data,iter=1,warmup=0,chains=1, 
    algorithm="Fixed_param", show_messages=F, open_progress=F, refresh=0)
  out <- rstan::extract(fit)
  extractor(out, stan_data) %>% mutate(sample_id=sample_id)
}

simulation_post_process <- function(simulated_df_list, initial_dataset){
  simulated_df_list %>%
    bind_rows() %>%
    # add region_id
    mutate(region_id = dense_rank(region)) %>%
    group_by(region) %>%
    # add dates
    group_modify(function(region_data, region_name){
      min_date <- initial_dataset %>% filter(region == region_name) %>% pull(date) %>% min()
      region_data %>% mutate(date = min_date + day_id - 1)
    }) %>% 
    # rerank sim ids
    mutate(sim_id = dense_rank(sample_id)) %>%
    # reorder columns
    select(sim_id, sample_id, region, region_id, date, day_id, everything()) %>%
    arrange(sim_id)
}
  
