suppressPackageStartupMessages({
  library(tidyverse)
  library(EnvStats) # required for pgammaAlt()
  library(pbapply)
  source("scripts/data_preproc/preproc_functions.R")
  source("scripts/data_preproc/stan-input-functions.R")
})


# ------------------------------------------------------------------------------
# STAN dataset preparation

fr_dataset <-
  read.csv("results/merge_spf_opencovid_agences_regionales_sante/mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset()

regions <- unique(fr_dataset$region) %>% sort()

covariates <- load_covariates("data/interventions.csv")

## Load french region population data
ifr_pop_data <- read.csv("data/region_wise_IFR.csv", sep=";") %>% 
  prepare_regions_pop_ifr()

grouped_dataset <- fr_dataset %>%
  group_by(region, region_id) %>%
  group_modify(function (region_data, keys) {
    message(keys$region %>% str_to_upper())
    message(" + PADDING : ")
    region_data <- pad_region_data(region_data)
    message(" + SLICING : ")
    region_data <- slice_from_onset(region_data)
  })

# Next step : convert to STAN input dataset
N2 = 150

stan_data_initial <- grouped_dataset %>% 
  group_map(make_region_stan_input, covariates, ifr_pop_data, N2) %>%
  # Merge region-specific data together
  reduce(merge_regions_stan_inputs, covariates) %>%
  # Add non-region-specific STAN inputs
  list_modify(
    K = 2, # Number of mixture components
    N_obs = sum(.$N), # number of days with observed data
    M = length(regions),
    N2 = N2, # total number of days (observed + missing + forecast)
    N0 = 6,  # N0 = 6 to make it consistent with Rayleigh
    SI = pgammaAlt(1:N2, 6.5, 0.62) - pgammaAlt((1:N2)-1, 6.5, 0.62),
    regions=regions,
    public_events=zap(),
    schools_universities=zap(),
    postlockdown=zap(),
    deaths_obs=zap(),
    mm=zap(),
    nn=zap(),
    is_weekend=zap(),
    elections=zap(),
    EpidemicStart=zap()
  )

# ------------------------------------------------------------------------------

source("scripts/simulations/sim_functions.R")
source("scripts/simulations/sim_mixture_functions.R")

pboptions(type="timer") 

loaded_data <- new.env()
load("results/region-sparse-oneintervention-mixture-gamma-prior-latest.Rdata",
  envir=loaded_data, verbose=T)
N_sim <- 200
stanfit <- loaded_data$out
samples <- sample(1:length(stanfit$phi), size=N_sim)
dataset <- loaded_data$grouped_dataset

LOCKDOWN <- ymd("2020-03-17")

shift_vector <- c(-7:7)

simulated_dataset <- shift_vector %>% 
  map_dfr(function(shift_n){
    sprintf("Lockdown shift: %d", shift_n) %>% message()
    stan_data <- shift_lockdown(stan_data_initial, shift_n)
    data_modifier <- make_mixture_stan_modifier(stan_data, stanfit)
    samples %>% 
      pblapply(mixture_simulation, data_modifier, cl=parallel::detectCores() - 1) %>%
      bind_rows() %>%
      mutate(lockdown_day = LOCKDOWN + shift_n)
  })  %>% 
  left_join(
    grouped_dataset %>% rename(deaths_obs=deaths) %>% select(-t), 
    by=c("region", "region_id", "day_id")) %>%
  group_by(region, region_id) %>%
  mutate(
    date=min(na.omit(date)) + day_id - 1,
    deaths_pred = rnbinom(deaths_mean, size = stanfit$phi[sample_id], mu=deaths_mean)) %>%
  rename(Rt=Rt_adj)


# save(simulated_dataset, grouped_dataset, file="results/mixture_lockdown_shift_explicit.Rdata")
data.table::fwrite(simulated_dataset, 
  file="results/simulated_datasets/mixture_lockdown_shift_explicit.csv")

