suppressPackageStartupMessages({
  library(rstan)
  library(optparse)
  library(tidyverse)
  library(EnvStats) # required for pgammaAlt()
  library(pbapply)
  library(ggpubr)
  source("scripts/data_preproc/preproc_functions.R")
  source("scripts/data_preproc/stan-input-functions.R")
})

parser <- OptionParser()

cmdoptions <- parse_args(parser,
  args = commandArgs(trailingOnly = TRUE),
  positional_arguments = TRUE)


# ------------------------------------------------------------------------------
# STAN dataset preparation
fr_dataset <-
  read.csv("results/merge_spf_opencovid_agences_regionales_sante/mergedDataReadyToAnalyze.csv", h=T) %>%
  prepare_french_region_dataset()

regions <- unique(fr_dataset$region) %>% sort()

covariates <- load_covariates("data/interventions.csv")

## Load french region population data
ifr_pop_data <-
  read.csv("data/region_wise_IFR.csv", sep=";") %>%
  prepare_regions_pop_ifr()

grouped_dataset <- fr_dataset %>%
  group_by(region, region_id) %>%
  group_modify(function (region_data, keys) {
    message(keys$region %>% str_to_upper())
    message(" + PADDING : ")
    region_data <- pad_region_data(region_data)
    message(" + SLICING : ")
    region_data <- slice_from_onset(region_data)
  })

# Next step : convert to STAN input dataset
N2 = 200

stan_data_initial <- grouped_dataset %>% 
  group_map(make_region_stan_input, covariates, ifr_pop_data, N2) %>%
  # Merge region-specific data together
  reduce(merge_regions_stan_inputs, covariates) %>%
  # Add non-region-specific STAN inputs
  list_modify(
    K = 2, # Number of mixture   components
    N_obs = sum(.$N), # number of days with observed data
    M = length(regions),
    N2 = N2, # total number of days (observed + missing + forecast)
    N0 = 6,  # N0 = 6 to make it consistent with Rayleigh
    SI = pgammaAlt(1:N2, 6.5, 0.62) - pgammaAlt((1:N2)-1, 6.5, 0.62),
    regions = regions
  )

if(stan_data_initial$N2 < max(stan_data_initial$nn)){
  sprintf("N2 = %d is insufficient to cover all %d observed days.", stan_data_initial$N2, max(stan_data_initial$nn)) %>% 
    paste(sprintf("Increase N2 to at least %d", max(stan_data_initial$nn))) %>% warning()
  stop("Invalid parameter")
}

# ------------------------------------------------------------------------------

source("scripts/simulations/sim_functions.R")
invisible(Sys.setlocale(locale="en_US.utf-8"))

rstan_options(auto_write = TRUE)
pboptions(type="txt")

# Compile STAN model
model_sim <- stan_model('models/simulations/simulation-no-interv.stan')

loaded_data <- new.env()
load("results/region-sparse-lockdown-latest.Rdata", env=loaded_data)
stanfit <- loaded_data$out
N_sim <- 200
samples <- sample(1:length(stanfit$phi), size=N_sim)

sprintf("Simulation size: %s", N_sim) %>% message()
sprintf("Number of days N2 : %s", N2) %>% message()

simulated_dataset <- pblapply(
  samples, simulation, model_sim, 
  make_default_stan_modifier(stan_data_initial, stanfit), 
  cl=parallel::detectCores() - 1) %>% 
  simulation_post_process(grouped_dataset)

output_table <- "results/simulated_datasets/simulated_data_no_interv.csv"
sprintf("Saving simulated data @ %s", output_table) %>% message()
data.table::fwrite(simulated_dataset, output_table)
